#!/usr/bin/python
# -*- coding:utf-8 -*-

import unittest
from tomato.store import MongoClient


class TestMongoClient(unittest.TestCase):

    def setUp(self):
        print('setUp...')

    def tearDown(self):
        print('tearDown...')

    def test_open(self):
        pass

    def test_insert(self):
        pass

    def test_update(self):
        pass

    def test_select(self):
        pass

    def test_close(self):
        pass
