#!/usr/bin/python
# -*- coding:utf-8 -*-


text = '''<html>
<head>
<title>Welcome to Tomato!</title>
<style>
    body {
        width: 35em;
        margin: 0 auto;
        font-family: Tahoma, Verdana, Arial, sans-serif;
    }
</style>
</head>
<body>
<h1>Welcome to Tomato!</h1>
<p>If you see this page, the tomato server is successfully installed and working.</p>
<p>For online documentation and support please refer to 
<a href="https://github.com/tangmi001/tomatolib/" target="_blank">tomatolib</a>.<br />

<p><em>Thank you for use tomatolib.</em></p>
</body>
</html>
'''
